==> workbench_approver-1.0.0.tar.gz: workbench_approver/tests/src/Functional/ApproverBlockTest.php <==
<?php

namespace Drupal\Tests\workbench_approver\Functional;

use Drupal\Tests\block\Traits\BlockCreationTrait;

/**
 * Tests for access to approval pages and tabs.
 *
 * @group workbench_approver
 */
class ApproverBlockTest extends WorkbenchApproverTestBase {

  use BlockCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'block',
    'workbench_approver',
  ];

  /**
   * A node with assignments and a published revision.
   *
   * @var \Drupal\node\NodeInterface
   */
  protected $assignedNode;

  /**
   * A node with assignments and no published revision.
   *
   * @var \Drupal\node\NodeInterface
   */
  protected $draftNode;

  /**
   * A node without assignments.
   *
   * @var \Drupal\node\NodeInterface
   */
  protected $nonAssignedNode;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $values = [
      'type' => 'approved',
      'title' => 'Assigned node',
      'moderation_state' => 'published',
    ];
    $this->assignedNode = $this->drupalCreateNode($values);
    $this->assignedNode = $this->createRevision($this->assignedNode);

    // Assign two users to approve this node.
    // Use a fixed timestamp for testing (Wed Feb 02 2035 02:00:37 GMT+0000)
    $approver_values = [
      'deadline' => 2053994437,
    ];
    $this->addApprovalRequest($this->adminUser, $this->assignedNode, $approver_values);
    $this->addApprovalRequest($this->editorUser, $this->assignedNode, $approver_values);

    $values = [
      'type' => 'approved',
      'title' => 'Draft node',
      'moderation_state' => 'draft',
    ];
    $this->draftNode = $this->drupalCreateNode($values);

    // Assign two users to approve this node.
    $this->addApprovalRequest($this->adminUser, $this->draftNode, $approver_values);
    $this->addApprovalRequest($this->editorUser, $this->draftNode, $approver_values);

    $values = [
      'type' => 'approved',
      'title' => 'Non-assigned node',
      'moderation_state' => 'published',
    ];
    $this->nonAssignedNode = $this->drupalCreateNode($values);
    $this->nonAssignedNode = $this->createRevision($this->nonAssignedNode);

    // Place our block.
    $settings = [
      'label' => 'Workbench approver form',
    ];
    $this->placeBlock('workbench_approver_block', $settings);
  }

  /**
   * Tests block visibility.
   *
   * @covers \Drupal\workbench_approver\Plugin\Block\ApproverBlock::access()
   */
  public function testApproverBlockAccess() {
    // Login each user and get expected results.
    $results = $this->getExpectedResults();

    $path = '/node/' . $this->assignedNode->id() . '/latest';
    $path2 = '/node/' . $this->nonAssignedNode->id() . '/latest';

    foreach ($results as $id => $expected) {
      // @phpstan-ignore-next-line
      $user = $this->{$id};
      $this->drupalLogin($user);
      $this->drupalGet($path);
      // Block based on permissions.
      if ($expected['view']) {
        $this->assertSession()->pageTextContains('Workbench approver form');
      }
      // No block because no approval requests.
      $this->drupalGet($path2);
      $this->assertSession()->pageTextNotContains('Workbench approver form');
    }
  }

  /**
   * Tests block information.
   *
   * @covers \Drupal\workbench_approver\Plugin\Block\ApproverBlock::build()
   */
  public function testApproverBlockInformation() {
    // Login each user and get expected results.
    $results = $this->getExpectedResults();

    $path = '/node/' . $this->assignedNode->id() . '/latest';
    foreach ($results as $id => $expected) {
      // @phpstan-ignore-next-line
      $user = $this->{$id};
      $this->drupalLogin($user);
      $this->drupalGet($path);
      if ($expected['assigned']) {
        $this->assertSession()->pageTextContains(sprintf('Your review of this revision has been requested by %s to be completed by %s.', $this->adminUser->label(), '02-Feb-2035'));
        $this->assertSession()->pageTextContains('This content may not be set to a new moderation state.');
      }
      else {
        $this->assertSession()->pageTextNotContains('Your review of this revision has been requested');
      }
      if ($expected['view']) {
        $this->assertSession()->pageTextContains('There are 2 outstanding approval requests for this revision.');
        $this->assertSession()->pageTextContains('This content may not be set to a new moderation state.');
      }
      else {
        $this->assertSession()->pageTextNotContains('There are 2 outstanding approval requests for this revision.');
        $this->assertSession()->pageTextNotContains('This content may not be set to a new moderation state.');
      }
    }
  }

  /**
   * Tests block form.
   *
   * @covers \Drupal\workbench_approver\Plugin\Block\ApproverBlock::build()
   */
  public function testApproverBlockForm() {
    // Login each user and get expected results.
    $results = $this->getExpectedResults();

    $path = '/node/' . $this->assignedNode->id() . '/latest';
    foreach ($results as $id => $expected) {
      // @phpstan-ignore-next-line
      $user = $this->{$id};
      $this->drupalLogin($user);
      $this->drupalGet($path);
      if ($expected['use_form']) {
        $this->assertSession()->buttonExists('Approve revision');
        $this->assertSession()->buttonExists('Request changes');
      }
      else {
        $this->assertSession()->buttonNotExists('Approve revision');
        $this->assertSession()->buttonNotExists('Request changes');
      }
    }

    // Now, as the admin user, approve this revision.
    $this->drupalLogin($this->adminUser);
    $this->drupalGet($path);
    $this->assertSession()->buttonExists('Approve revision')->click();
    // AJAX cancel action.
    $this->assertSession()->buttonExists('Cancel')->click();
    $this->assertSession()->addressEquals($path);

    // This is the ajax form click.
    $this->assertSession()->buttonExists('Approve revision')->click();
    $date = date('d-M-Y', $this->container->get('datetime.time')->getRequestTime());
    $this->assertSession()->buttonExists('Approve revision')->click();
    $this->assertSession()->pageTextNotContains('Your review of this revision has been requested');
    $this->assertSession()->pageTextContains(sprintf('You Approved this revision on %s.', $date));
    $this->assertSession()->pageTextContains('There is 1 outstanding approval request for this revision.');
    $this->assertSession()->buttonNotExists('Approve revision');
    $this->assertSession()->buttonNotExists('Request changes');
    $this->assertSession()->addressEquals($path);

    // Check as the editorUser.
    $this->drupalLogin($this->editorUser);
    $this->drupalGet($path);
    $this->assertSession()->buttonExists('Approve revision');
    $this->assertSession()->buttonExists('Request changes');
    $this->assertSession()->pageTextContains(sprintf('Your review of this revision has been requested by %s to be completed by %s.', $this->adminUser->label(), '02-Feb-2035'));
    $this->assertSession()->pageTextNotContains('You Approved this revision');
    $this->assertSession()->pageTextContains('There is 1 outstanding approval request for this revision.');
  }

  /**
   * Tests block visibility on nodes without a published version.
   *
   * @covers \Drupal\workbench_approver\Plugin\Block\ApproverBlock::build()
   */
  public function testApproverBlockUnpublishedNode() {
    // Login each user and get expected results.
    $results = $this->getExpectedResults();

    $path = '/node/' . $this->draftNode->id();

    foreach ($results as $id => $expected) {
      // @phpstan-ignore-next-line
      $user = $this->{$id};
      $this->drupalLogin($user);
      $this->drupalGet($path);
      // Block based on permissions.
      if ($expected['view']) {
        $this->assertSession()->pageTextContains('Workbench approver form');
      }
    }
  }

  /**
   * Tests the block does not appear on edit or delete paths.
   */
  public function testApproverBlockRoutes() {
    // Login each user and get expected results.
    $this->drupalLogin($this->adminUser);

    $paths = [
      '/node/' . $this->draftNode->id() . '/edit',
      '/node/' . $this->draftNode->id() . '/delete',
    ];

    foreach ($paths as $path) {
      $this->drupalGet($path);
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->draftNode->label());
      $this->assertSession()->pageTextNotContains('Workbench approver form');
    }
  }

  /**
   * Returns an array of expected checks.
   *
   * @return array
   *   An array of settings for the tests.
   */
  private function getExpectedResults(): array {
    return [
      'adminUser' => [
        'view' => TRUE,
        'assigned' => TRUE,
        'use_form' => TRUE,
      ],
      'editorUser' => [
        'view' => TRUE,
        'assigned' => TRUE,
        'use_form' => TRUE,
      ],
      'observerUser' => [
        'view' => FALSE,
        'assigned' => FALSE,
        'use_form' => FALSE,
      ],
      'workflowAdminUser' => [
        'view' => TRUE,
        'assigned' => FALSE,
        'use_form' => FALSE,
      ],
    ];
  }

}

==> workbench_approver-1.0.1.tar.gz: workbench_approver/tests/src/Functional/ApproverBlockTest.php <==
<?php

namespace Drupal\Tests\workbench_approver\Functional;

use Drupal\Tests\block\Traits\BlockCreationTrait;

/**
 * Tests for access to approval pages and tabs.
 *
 * @group workbench_approver
 */
class ApproverBlockTest extends WorkbenchApproverTestBase {

  use BlockCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'block',
    'workbench_approver',
  ];

  /**
   * A node with assignments and a published revision.
   *
   * @var \Drupal\node\NodeInterface
   */
  protected $assignedNode;

  /**
   * A node with assignments and no published revision.
   *
   * @var \Drupal\node\NodeInterface
   */
  protected $draftNode;

  /**
   * A node without assignments.
   *
   * @var \Drupal\node\NodeInterface
   */
  protected $nonAssignedNode;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $values = [
      'type' => 'approved',
      'title' => 'Assigned node',
      'moderation_state' => 'published',
    ];
    $this->assignedNode = $this->drupalCreateNode($values);
    $this->assignedNode = $this->createRevision($this->assignedNode);

    // Assign two users to approve this node.
    // Use a fixed timestamp for testing (Wed Feb 02 2035 02:00:37 GMT+0000)
    $approver_values = [
      'deadline' => 2053994437,
    ];
    $this->addApprovalRequest($this->adminUser, $this->assignedNode, $approver_values);
    $this->addApprovalRequest($this->editorUser, $this->assignedNode, $approver_values);

    $values = [
      'type' => 'approved',
      'title' => 'Draft node',
      'moderation_state' => 'draft',
    ];
    $this->draftNode = $this->drupalCreateNode($values);

    // Assign two users to approve this node.
    $this->addApprovalRequest($this->adminUser, $this->draftNode, $approver_values);
    $this->addApprovalRequest($this->editorUser, $this->draftNode, $approver_values);

    $values = [
      'type' => 'approved',
      'title' => 'Non-assigned node',
      'moderation_state' => 'published',
    ];
    $this->nonAssignedNode = $this->drupalCreateNode($values);
    $this->nonAssignedNode = $this->createRevision($this->nonAssignedNode);

    // Place our block.
    $settings = [
      'label' => 'Workbench Approver form',
    ];
    $this->placeBlock('workbench_approver_block', $settings);
  }

  /**
   * Tests block visibility.
   *
   * @covers \Drupal\workbench_approver\Plugin\Block\ApproverBlock::access()
   */
  public function testApproverBlockAccess() {
    // Login each user and get expected results.
    $results = $this->getExpectedResults();

    $path = '/node/' . $this->assignedNode->id() . '/latest';
    $path2 = '/node/' . $this->nonAssignedNode->id() . '/latest';

    foreach ($results as $id => $expected) {
      // @phpstan-ignore-next-line
      $user = $this->{$id};
      $this->drupalLogin($user);
      $this->drupalGet($path);
      // Block based on permissions.
      if ($expected['view']) {
        $this->assertSession()->pageTextContains('Workbench Approver form');
      }
      // No block because no approval requests.
      $this->drupalGet($path2);
      $this->assertSession()->pageTextNotContains('Workbench Approver form');
    }
  }

  /**
   * Tests block information.
   *
   * @covers \Drupal\workbench_approver\Plugin\Block\ApproverBlock::build()
   */
  public function testApproverBlockInformation() {
    // Login each user and get expected results.
    $results = $this->getExpectedResults();

    $path = '/node/' . $this->assignedNode->id() . '/latest';
    foreach ($results as $id => $expected) {
      // @phpstan-ignore-next-line
      $user = $this->{$id};
      $this->drupalLogin($user);
      $this->drupalGet($path);
      if ($expected['assigned']) {
        $this->assertSession()->pageTextContains(sprintf('Your review of this revision has been requested by %s to be completed by %s.', $this->adminUser->label(), '02-Feb-2035'));
        $this->assertSession()->pageTextContains('This content may not be set to a new moderation state.');
      }
      else {
        $this->assertSession()->pageTextNotContains('Your review of this revision has been requested');
      }
      if ($expected['view']) {
        $this->assertSession()->pageTextContains('There are 2 outstanding approval requests for this revision.');
        $this->assertSession()->pageTextContains('This content may not be set to a new moderation state.');
      }
      else {
        $this->assertSession()->pageTextNotContains('There are 2 outstanding approval requests for this revision.');
        $this->assertSession()->pageTextNotContains('This content may not be set to a new moderation state.');
      }
    }
  }

  /**
   * Tests block form.
   *
   * @covers \Drupal\workbench_approver\Plugin\Block\ApproverBlock::build()
   */
  public function testApproverBlockForm() {
    // Login each user and get expected results.
    $results = $this->getExpectedResults();

    $path = '/node/' . $this->assignedNode->id() . '/latest';
    foreach ($results as $id => $expected) {
      // @phpstan-ignore-next-line
      $user = $this->{$id};
      $this->drupalLogin($user);
      $this->drupalGet($path);
      if ($expected['use_form']) {
        $this->assertSession()->buttonExists('Approve revision');
        $this->assertSession()->buttonExists('Request changes');
      }
      else {
        $this->assertSession()->buttonNotExists('Approve revision');
        $this->assertSession()->buttonNotExists('Request changes');
      }
    }

    // Now, as the admin user, approve this revision.
    $this->drupalLogin($this->adminUser);
    $this->drupalGet($path);
    $this->assertSession()->buttonExists('Approve revision')->click();
    // AJAX cancel action.
    $this->assertSession()->buttonExists('Cancel')->click();
    $this->assertSession()->addressEquals($path);

    // This is the ajax form click.
    $this->assertSession()->buttonExists('Approve revision')->click();
    $date = date('d-M-Y', $this->container->get('datetime.time')->getRequestTime());
    $this->assertSession()->buttonExists('Approve revision')->click();
    $this->assertSession()->pageTextNotContains('Your review of this revision has been requested');
    $this->assertSession()->pageTextContains(sprintf('You Approved this revision on %s.', $date));
    $this->assertSession()->pageTextContains('There is 1 outstanding approval request for this revision.');
    $this->assertSession()->buttonNotExists('Approve revision');
    $this->assertSession()->buttonNotExists('Request changes');
    $this->assertSession()->addressEquals($path);

    // Check as the editorUser.
    $this->drupalLogin($this->editorUser);
    $this->drupalGet($path);
    $this->assertSession()->buttonExists('Approve revision');
    $this->assertSession()->buttonExists('Request changes');
    $this->assertSession()->pageTextContains(sprintf('Your review of this revision has been requested by %s to be completed by %s.', $this->adminUser->label(), '02-Feb-2035'));
    $this->assertSession()->pageTextNotContains('You Approved this revision');
    $this->assertSession()->pageTextContains('There is 1 outstanding approval request for this revision.');
  }

  /**
   * Tests block visibility on nodes without a published version.
   *
   * @covers \Drupal\workbench_approver\Plugin\Block\ApproverBlock::build()
   */
  public function testApproverBlockUnpublishedNode() {
    // Login each user and get expected results.
    $results = $this->getExpectedResults();

    $path = '/node/' . $this->draftNode->id();

    foreach ($results as $id => $expected) {
      // @phpstan-ignore-next-line
      $user = $this->{$id};
      $this->drupalLogin($user);
      $this->drupalGet($path);
      // Block based on permissions.
      if ($expected['view']) {
        $this->assertSession()->pageTextContains('Workbench Approver form');
      }
    }
  }

  /**
   * Tests the block does not appear on edit or delete paths.
   */
  public function testApproverBlockRoutes() {
    // Login each user and get expected results.
    $this->drupalLogin($this->adminUser);

    $paths = [
      '/node/' . $this->draftNode->id() . '/edit',
      '/node/' . $this->draftNode->id() . '/delete',
    ];

    foreach ($paths as $path) {
      $this->drupalGet($path);
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->draftNode->label());
      $this->assertSession()->pageTextNotContains('Workbench Approver form');
    }
  }

  /**
   * Returns an array of expected checks.
   *
   * @return array
   *   An array of settings for the tests.
   */
  private function getExpectedResults(): array {
    return [
      'adminUser' => [
        'view' => TRUE,
        'assigned' => TRUE,
        'use_form' => TRUE,
      ],
      'editorUser' => [
        'view' => TRUE,
        'assigned' => TRUE,
        'use_form' => TRUE,
      ],
      'observerUser' => [
        'view' => FALSE,
        'assigned' => FALSE,
        'use_form' => FALSE,
      ],
      'workflowAdminUser' => [
        'view' => TRUE,
        'assigned' => FALSE,
        'use_form' => FALSE,
      ],
    ];
  }

}
